@@ -24,8 +24,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "getopt.h"
 
 
-extern char *version_string;
-
 extern void print_variable_data_base ();
 extern void print_dir_data_base ();
 extern void print_rule_data_base ();
@@ -391,7 +389,6 @@ main (argc, argv, envp)
   register char *cmd_defs;
   register unsigned int cmd_defs_len, cmd_defs_idx;
   char **p;
-  time_t now;
   struct dep *goals = 0;
   register struct dep *lastgoal;
   struct dep *read_makefiles;
@@ -498,15 +495,26 @@ main (argc, argv, envp)
   for (i = 0; envp[i] != 0; ++i)
     {
       register char *ep = envp[i];
-      while (*ep++ != '=')
-	;
-      (void) define_variable (envp[i], ep - envp[i] - 1, ep, o_env, 1);
+      while (*ep != '=')
+	++ep;
+      define_variable (envp[i], ep - envp[i], ep + 1, o_env, 1)
+	/* Force exportation of every variable culled from the environment.
+	   We used to rely on target_environment's v_default code to do this.
+	   But that does not work for the case where an environment variable
+	   is redefined in a makefile with `override'; it should then still
+	   be exported, because it was originally in the environment.  */
+	->export = v_export;
     }
 
   /* Decode the switches.  */
 
   decode_env_switches ("MAKEFLAGS", 9);
+#if 0
+  /* People write things like:
+     	MFLAGS="CC=gcc -pipe" "CFLAGS=-g"
+     and we set the -p, -i and -e switches.  Doesn't seem quite right.  */
   decode_env_switches ("MFLAGS", 6);
+#endif
   decode_switches (argc, argv, 0);
 
   /* Print version information.  */
@@ -553,7 +561,7 @@ main (argc, argv, envp)
 	  
 	  while (*p != '\0')
 	    {
-	      if (index (";'\"*?[]$<>(){}|&~`\\ \t\r\n\f\v", *p) != 0)
+	      if (index ("^;'\"*?[]$<>(){}|&~`\\ \t\r\n\f\v", *p) != 0)
 		cmd_defs[cmd_defs_idx++] = '\\';
 	      cmd_defs[cmd_defs_idx++] = *p++;
 	    }
@@ -738,6 +746,14 @@ main (argc, argv, envp)
 
   set_default_suffixes ();
 
+  /* Define the file rules for the built-in suffix rules.  These will later
+     be converted into pattern rules.  We used to do this in
+     install_default_implicit_rules, but since that happens after reading
+     makefiles, it results in the built-in pattern rules taking precedence
+     over makefile-specified suffix rules, which is wrong.  */
+
+  install_default_suffix_rules ();
+
   /* Define some internal and special variables.  */
 
   define_automatic_variables ();
@@ -759,7 +775,9 @@ main (argc, argv, envp)
 
   /* Decode switches again, in case the variables were set by the makefile.  */
   decode_env_switches ("MAKEFLAGS", 9);
+#if 0
   decode_env_switches ("MFLAGS", 6);
+#endif
 
   /* Set up MAKEFLAGS and MFLAGS again, so they will be right.  */
 
@@ -774,17 +792,20 @@ main (argc, argv, envp)
 
   snap_deps ();
 
-  /* Install the default implicit rules.
+  /* Convert old-style suffix rules to pattern rules.  It is important to
+     do this before installing the built-in pattern rules below, so that
+     makefile-specified suffix rules take precedence over built-in pattern
+     rules.  */
+
+  convert_to_pattern ();
+
+  /* Install the default implicit pattern rules.
      This used to be done before reading the makefiles.
      But in that case, built-in pattern rules were in the chain
      before user-defined ones, so they matched first.  */
 
   install_default_implicit_rules ();
 
-  /* Convert old-style suffix rules to pattern rules.  */
-
-  convert_to_pattern ();
-
   /* Compute implicit rule limits.  */
 
   count_implicit_rule_limits ();
@@ -794,8 +815,8 @@ main (argc, argv, envp)
   build_vpath_lists ();
 
   /* Mark files given with -o flags as very old (00:00:01.00 Jan 1, 1970)
-     and as having been updated already, and files given with -W flags
-     as brand new (time-stamp of now).  */
+     and as having been updated already, and files given with -W flags as
+     brand new (time-stamp as far as possible into the future).  */
 
   if (old_files != 0)
     for (p = old_files->list; *p != 0; ++p)
@@ -809,11 +830,10 @@ main (argc, argv, envp)
 
   if (new_files != 0)
     {
-      now = time ((time_t *) 0);
       for (p = new_files->list; *p != 0; ++p)
 	{
 	  f = enter_command_line_file (*p);
-	  f->last_mtime = now;
+	  f->last_mtime = NEW_MTIME;
 	}
     }
 
@@ -1638,14 +1658,24 @@ define_makeflags (all, makefile)
       *p = '\0';
     }
 
-  (void) define_variable ("MAKEFLAGS", 9,
-			  /* On Sun, the value of MFLAGS starts with a `-' but
-			     the value of MAKEFLAGS lacks the `-'.
-			     Be compatible with this unless FLAGSTRING starts
-			     with a long option `--foo', since removing the
-			     first dash would result in the bogus `-foo'.  */
-			  flagstring[1] == '-' ? flagstring : &flagstring[1],
-			  o_env, 0);
+  define_variable ("MAKEFLAGS", 9,
+		   /* On Sun, the value of MFLAGS starts with a `-' but
+		      the value of MAKEFLAGS lacks the `-'.
+		      Be compatible with this unless FLAGSTRING starts
+		      with a long option `--foo', since removing the
+		      first dash would result in the bogus `-foo'.  */
+		   flagstring[1] == '-' ? flagstring : &flagstring[1],
+		   /* This used to use o_env, but that lost when a
+		      makefile defined MAKEFLAGS.  Makefiles set
+		      MAKEFLAGS to add switches, but we still want
+		      to redefine its value with the full set of
+		      switches.  Of course, an override or command
+		      definition will still take precedence.  */
+		   o_file, 0)
+    /* Always export MAKEFLAGS.  */
+    ->export = v_export;
+  /* Since MFLAGS is not parsed for flags, there is no reason to
+     override any makefile redefinition.  */
   (void) define_variable ("MFLAGS", 6, flagstring, o_env, 0);
 }
 
@@ -1656,7 +1686,6 @@ print_version ()
 {
   static int printed_version = 0;
 
-  extern char *remote_description;
   char *precede = print_data_base_flag ? "# " : "";
 
   if (printed_version)
